<?php
namespace Teknisa\COZ\Services;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\COZ\Services\ControllerWrapper;
use Teknisa\GER\Controllers\Grupfili;
use Teknisa\COZ\Controllers\Filiaisagregadas;
use Teknisa\COZControllers\Grupserv;
use Teknisa\COZ\Controllers\ServicoPorFiliais;
use Teknisa\COZControllers\Tipoprat;
use Teknisa\COZControllers\Setorcoz;
use Teknisa\Model\Planejamento\Entities\Planproducao;
use Teknisa\COZ\Controllers\Pla10500CancelaPlanproducao;
use Teknisa\COZ\Controllers\Pla10500TipopratoNovaReceita;
use Teknisa\COZ\Controllers\Pla10500ValoresProdutoNovaReceita;
use Teknisa\COZ\Controllers\Pla10500Produzidos;
use Teknisa\COZ\Controllers\Pla10500DataFinalAdiantamento;
use Teknisa\COZ\Controllers\Pla10500ProdutoJaProduzido;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class Pla10500RemoteRepository{

    protected $grupfili;
    protected $filiaisagregadas;
    protected $grupserv;
    protected $servico_por_filiais;
    protected $tipoprat;
    protected $setorcoz;
    protected $pla10500_planproducao;
    protected $pla10500_cancela_planproducao;
    protected $pla10500_retorna_tipoprato_nova_receita;
    protected $pla10500_retorna_valores_produto_nova_receita;
    protected $pla10500_produzidos;
    protected $pla10500_data_final_adiantamento;
    protected $pla10500_produto_ja_produzido;
    protected $pla10500_produtos_produzidos;
    protected $pla10500verificaEstProdRecProdAcabado;
    protected $pla10500verificaEstProdRecProdAcabadoReserva;
    protected $logmatprima;

    public function __construct(
        $grupfili, $filiaisagregadas, $grupserv, $servico_por_filiais,
        $tipoprat, $setorcoz, $pla10500_planproducao, $pla10500_cancela_planproducao,
        $pla10500_retorna_tipoprato_nova_receita, $pla10500_retorna_valores_produto_nova_receita,
        $pla10500_produzidos, $pla10500_data_final_adiantamento, $pla10500_produto_ja_produzido,
        $pla10500_produtos_produzidos, $pla10500verificaEstProdRecProdAcabado, $logmatprima,
        $pla10500verificaEstProdRecProdAcabadoReserva
    ){
        $this->grupfili = $grupfili;
        $this->filiaisagregadas = $filiaisagregadas;
        $this->grupserv = $grupserv;
        $this->servico_por_filiais = $servico_por_filiais;
        $this->tipoprat = $tipoprat;
        $this->setorcoz = $setorcoz;
        $this->pla10500_planproducao = $pla10500_planproducao;
        $this->pla10500_cancela_planproducao = $pla10500_cancela_planproducao;
        $this->pla10500_retorna_tipoprato_nova_receita = $pla10500_retorna_tipoprato_nova_receita;
        $this->pla10500_retorna_valores_produto_nova_receita = $pla10500_retorna_valores_produto_nova_receita;
        $this->pla10500_produzidos = $pla10500_produzidos;
        $this->pla10500_data_final_adiantamento = $pla10500_data_final_adiantamento;
        $this->pla10500_produto_ja_produzido = $pla10500_produto_ja_produzido;
        $this->pla10500_produtos_produzidos = $pla10500_produtos_produzidos;
        $this->pla10500verificaEstProdRecProdAcabado = $pla10500verificaEstProdRecProdAcabado;
        $this->logmatprima = $logmatprima;
        $this->pla10500verificaEstProdRecProdAcabadoReserva = $pla10500verificaEstProdRecProdAcabadoReserva;
    }

    public function fetchGrupfili(){
        $emptyFilter = new FilterCriteria("", array());
        $rows = ControllerWrapper::fetchFromController($this->grupfili, $emptyFilter);
        return array_column($rows, "CDGRUPFILI");
    }

    public function fetchFiliaisagregadas($params){
        $filterCriteria = new FilterCriteria("filiaisagregadas", $params);
        $rows = ControllerWrapper::fetchFromController($this->filiaisagregadas, $filterCriteria);
        return array_column($rows, "CDFILIAL");
    }

    public function fetchGrupserv(){
        $emptyFilter = new FilterCriteria("", array());
        $rows = ControllerWrapper::fetchFromController($this->grupserv, $emptyFilter);
        return array_column($rows, "CDGRUPSERV");
    }

    public function fetchServicos($params){
        $filterCriteria = new FilterCriteria("servico_por_filiais", $params);
        $rows = ControllerWrapper::fetchFromController($this->servico_por_filiais, $filterCriteria);
        return array_map(function($item){
            return array("CDFILIAL" => $item['CDFILIAL'], "CDSERVFILI" => $item['CDSERVFILI'],
                "SERVICO" => $item['CDFILIAL'].$item['CDSERVFILI']);
        }, $rows);
    }

    public function fetchTipoPrato(){
        $emptyFilter = new FilterCriteria("", array());
        $rows = ControllerWrapper::fetchFromController($this->tipoprat, $emptyFilter);
        return array_column($rows, "CDTIPOPRAT");
    }

    public function fetchSetorcoz(){
        $emptyFilter = new FilterCriteria("", array());
        $rows = ControllerWrapper::fetchFromController($this->setorcoz, $emptyFilter);
        return array_column($rows, "CDSETORCOZ");
    }

    public function fetchPlanProducao($params){
        $filterCriteria = new FilterCriteria("pla10500_planproducao", $params);
        $rows = ControllerWrapper::fetchFromController($this->pla10500_planproducao, $filterCriteria);
        return array_map(function($item){
            return array("NRPLANPROD" => $item['NRPLANPROD'], "CDFILIAL" => $item['CDFILIAL'],
                        "DTPRODUCAO" => $item['DTPRODUCAO'], "DTINICARD" => $item['DTINICARD'],
                        "DTFINCARD" => $item['DTFINCARD'], "DTFINFILTRO" => $item['DTFINFILTRO'],
                        "IDRETIRADA" => $item['IDRETIRADA']);
        }, $rows);
    }

    public function fetchCancelaPlanProducao($params){
        $filterCriteria = new FilterCriteria("pla10500_cancela_planproducao", $params);
        $rows = ControllerWrapper::fetchFromController($this->pla10500_cancela_planproducao, $filterCriteria);
        return array_map(function($item){
            return array("IDCONCLUIDO" => $item['IDCONCLUIDO']);
        }, $rows);
    }

    public function fetchTipopratoNovaReceita($params){
        $filterCriteria = new FilterCriteria("pla10500_tipoprato_nova_receita", $params);
        $rows = ControllerWrapper::fetchFromController($this->pla10500_retorna_tipoprato_nova_receita, $filterCriteria);
        return array_map(function($item){
            return array("CDTIPOPRAT" => $item['CDTIPOPRAT']);
        }, $rows);
    }

    public function fetchValoresProdutoNovaReceita($params){
        $filterCriteria = new FilterCriteria("pla10500_valores_produto_nova_receita", $params);
        $rows = ControllerWrapper::fetchFromController($this->pla10500_retorna_valores_produto_nova_receita, $filterCriteria);
        return array_map(function($item){
            return array("VRUNITRECEPRAT" => $item['VRUNITRECEPRAT'], "VRTOTALRECEPRAT" => $item['VRTOTALRECEPRAT']);
        }, $rows);
    }

    public function fetchFilhosProduzidos($params){
        $filterCriteria = new FilterCriteria("pla10500_produzidos", $params);
        $rows = ControllerWrapper::fetchFromController($this->pla10500_produzidos, $filterCriteria);
        return array_map(function($item){
            return array("PRODUZIDOS" => $item['PRODUZIDOS']);
        }, $rows);
    }

    public function fetchDataFinal($params){
        $filterCriteria = new FilterCriteria("pla10500_data_final_adiantamento", $params);
        $rows = ControllerWrapper::fetchFromController($this->pla10500_data_final_adiantamento, $filterCriteria);
        return array_column($rows, "DTFINAL");
    }

    public function fetchProdutoJaProduzido($params){
        $filterCriteria = new FilterCriteria("pla10500_produto_ja_produzido", $params);
        $rows = ControllerWrapper::fetchFromController($this->pla10500_produto_ja_produzido, $filterCriteria);
        return array_column($rows, "PRODUZIDO");
    }

    public function fetchProdutosProduzidos($params){
        $filterCriteria = new FilterCriteria("pla10500_produtos_produzidos", $params);
        $rows = ControllerWrapper::fetchFromController($this->pla10500_produtos_produzidos, $filterCriteria);
        return array_column($rows, "PRODUZIDO");
    }

    public function verificaEstoqueProdutosReceita(
        $nrorg, $central, $nrPlanprod, $lista, $data
    ){
        $filterCriteria = new FilterCriteria("verificaEstProdRecProdAcabado");
        $filterCriteria->addCondition('NRORG', $nrorg);
        $filterCriteria->addCondition('CDCENTRAL', $central);
        $filterCriteria->addCondition('NRPLANPROD', $nrPlanprod);
        $filterCriteria->addCondition('LISTAPROD', $lista);
        $filterCriteria->addCondition('DTPRODUCAO', $data);
        $rows = ControllerWrapper::fetchFromController($this->pla10500verificaEstProdRecProdAcabado, $filterCriteria);
        return $rows;
    }

    public function salvaLogMP(
        $nrorg, $nrPlanprod, $codigo, $origem, $produtos, $prodSaldoInsuf, $operador
    ){
        $filterCriteria = new FilterCriteria("logmatprima");
        $filterCriteria->addCondition('NRORG', $nrorg);
        $rows = ControllerWrapper::fetchFromController($this->logmatprima, $filterCriteria);

        if ($rows) {
            foreach ($rows as $row) {
                if ($row['DSORIGEM'] === $origem and $row['DSJUSTIFICATIVA'] == '') {
                    $row['__is_new'] = false;

                    $request = new DTO\Request\Row(
                        $row, "delete", "/dummy", " "
                    );
                    $response = new DTO\Response();
                    $this->logmatprima->delete($request, $response);
                }
            }
        }

        $data = date('d/m/Y h:m:s');

        $connection = InstanceManager::getInstance()->getService('entityManager')->getConnection();
        foreach ($prodSaldoInsuf as $prod) {
            $dados = array(
                'NRLOGMATPRIMA' => $codigo,
                'CDPRODUTO' => $prod['CDPRODUTO'],
                'NRPLANPROD' => $nrPlanprod,
                'QTREQUERIDA' => $prod['QTNECESSARIA'],
                'QTESTOQUE' => $prod['QTESTDIACON'],
                'DTVALIDACAO' => $data,
                'DSORIGEM' => $origem,
                'DSSTRPRODACAB' => substr($produtos, 0, 1999),
                'DSJUSTIFICATIVA' => '',
                'NRORG' => $nrorg,
                'DTINCLUSAO' => $data,
                'NRORGINCLUSAO' => $nrorg,
                'CDOPERINCLUSAO' => $operador
            );

            $insert = "INSERT INTO LOGMATPRIMA (NRLOGMATPRIMA, CDPRODUTO, NRPLANPROD, QTREQUERIDA, QTESTOQUE,
                                           DTVALIDACAO, DSORIGEM, DSSTRPRODACAB, DSJUSTIFICATIVA, NRORG,
                                           DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO)
                                   VALUES (:NRLOGMATPRIMA, :CDPRODUTO, :NRPLANPROD, :QTREQUERIDA, :QTESTOQUE,
                                           :DTVALIDACAO, :DSORIGEM, :DSSTRPRODACAB, :DSJUSTIFICATIVA, :NRORG,
                                           :DTINCLUSAO, :NRORGINCLUSAO, :CDOPERINCLUSAO)";
            $sth = $connection->prepare($insert);
            $sth->execute($dados);
        }

        return true;
    }

    public function verificaEstoqueProdutosReceitaComReserva(
        $nrorg, $central, $nrPlanprod, $lista, $data
    ){
        $filterCriteria = new FilterCriteria("verificaEstProdRecProdAcabadoReserva");
        $filterCriteria->addCondition('LISTAPROD', $lista);
        $filterCriteria->addCondition('DTPRODUCAO', $data);
        $filterCriteria->addCondition('NRPLANPROD', $nrPlanprod);
        $filterCriteria->addCondition('NRORG', $nrorg);
        $filterCriteria->addCondition('CDCENTRAL', $central);
        $rows = ControllerWrapper::fetchFromController($this->pla10500verificaEstProdRecProdAcabadoReserva, $filterCriteria);
        return $rows;
    }
}
